#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "font.h"



typedef unsigned char	U08;			/* data type definition */
typedef   signed char	S08;
typedef unsigned int	U16;
typedef   signed int	S16;
typedef unsigned long	U32;
typedef   signed long	S32;


unsigned char ScreenMode = 'P';			// screen mode(P=portrait, L=landscape)
unsigned char LineLimit = 30;			// character length of line (30 or 40)
unsigned char KoreanBuffer[32] = {0};		// 32 byte Korean font buffer
unsigned char xcharacter, ycharacter;		// xcharacter(0-29), ycharacter(0-39) for portrait
						// xcharacter(0-39), ycharacter(0-29) for landscape
unsigned int  foreground, background;		// foreground and background color
unsigned char cursor_flag = 0;			// 0 = cursor off, 1 = cursor on
unsigned char xcursor, ycursor;			// cursor position
unsigned int  cursor;				// cursor color
unsigned char outline_flag = 0;			// 0 = outline off, 1 = outline on
unsigned int  outline;				// outline color


#define BLACK        0x0000		// RRRRR GGGGGG BBBBB   => 565
#define WHITE        0xffff
#define YELLOW       0xffe0
#define DARK_YELLOW  YELLOW & GRAY09
#define RED          0xf800
#define DARK_RED     RED & GRAY09
#define GREEN        0x07e0
#define DARK_GREEN   GREEN & GRAY09
#define BLUE         0x001f
#define DARK_BLUE    BLUE & GRAY09
#define CYAN         0x07ff
#define MAGENTA      0xf81f

#define GRAY01       0xef7d
#define GRAY02       0xe71c
#define GRAY03       0xd69a
#define GRAY04       0xc618
#define GRAY05       0xb596
#define GRAY06       0xa514
#define GRAY07       0x9492
#define GRAY08       0x738e
#define GRAY09       0x630c
#define GRAY10       0x528a
#define GRAY11       0x39e7
#define GRAY12       0x3186
#define GRAY13       0x2104
#define GRAY14       0x1082

#define Magenta		0xF81F			// Magenta = Red + Blue
#define Brown		0xA145
#define Khaki		0xF731
#define Olive		0x8400
#define Orange		0xFD20
#define Pink		0xFB56
#define Purple		0x8010
#define Silver		0xC618
#define Violet		0xEC1D
#define Transparent	0x0821




#define MAX_DOT      96000 // 240 X RGB X 400 dot
#define X_MAX_PIXEL  400
#define Y_MAX_PIXEL  240

// ȭ ǥ 
#define		SCREEN_COLUMN_SET	0			// ι ǥø ʻ(0,0) 
#define		SCREEN_ROW_SET		1			// ι ǥø ʻ(0,0) 

#define		SCREEN_COLUMN_X	399			// ι ǥø X ִ밪
#define		SCREEN_COLUMN_Y	239			// ι ǥø Y ִ밪
#define		SCREEN_ROW_X		239			// ι ǥø X ִ밪
#define		SCREEN_ROW_Y		399			// ι ǥø Y ִ밪

#define     RD_H    PORTA |= _BV(4);  // RD H
#define     RD_L	PORTA &= ~_BV(4); // 
#define     WR_H	PORTA |= _BV(5);  //
#define     WR_L    PORTA &= ~_BV(5); // 
#define     RS_H	PORTA |= _BV(6);  //
#define     RS_L    PORTA &= ~_BV(6); // 
#define     CS_H	PORTA |= _BV(7);  //
#define     CS_L    PORTA &= ~_BV(7); // 

#define     RST_H	PORTA |= _BV(2);  //reset H
#define     RST_L   PORTA &= ~_BV(2);// reset L

#define     BL_H	PORTA |= _BV(3);  //Ʈ  H
#define     BL_L    PORTA &= ~_BV(3);// Ʈ L


extern prog_uchar bird0[2960];	
	
 // 
int screen_mode = SCREEN_COLUMN_SET;				// ȭ ǥù 
int screen_xmax = SCREEN_COLUMN_X;				// θ ִ밪
int screen_ymax = SCREEN_COLUMN_Y;				// θ ִ밪
unsigned screen_color = RED;				// ڻ   WHITE;

unsigned screen_bkcolor = BLACK;				//  
int screen_curx = 0;						// X ǥ 
int screen_cury = 0;						// Yǥ 

const unsigned int line[1]= {YELLOW};
const unsigned int basic_color[8] = {BLACK, WHITE, RED, YELLOW, CYAN, BLUE, GREEN, MAGENTA};
const unsigned int gray16[16]= {0x0000, 0x1082, 0x2104, 0x3186, 0x39e7, 0x528a, 0x630c, 0x738e
                              , 0x9492, 0xa514, 0xb596, 0xc618, 0xd69a, 0xe71c, 0xef7d, 0xffff};

unsigned int d1,d1_1,d2,d2_1;

void delay_us(unsigned char time_us)		// delay for us	
{ register unsigned char i;

  for(i = 0; i < time_us; i++)			// 4 cycle +
    { asm volatile(" PUSH  R0 ");		// 2 cycle +
      asm volatile(" POP   R0 ");		// 2 cycle +
   	 asm volatile(" PUSH  R0 ");		// 2 cycle +
     asm volatile(" POP   R0 ");		// 2 cycle +
	 asm volatile(" PUSH  R0 ");		// 2 cycle +
      asm volatile(" POP   R0 ");		// 2 cycle = 16 cycle = 1 us for 16MHz
    }
}

void delay_ms(unsigned int time_ms)		// delay for ms	
{ register unsigned int i;

  for(i = 0; i < time_ms; i++)
    { delay_us(250);
      delay_us(250);
      delay_us(250);
      delay_us(250);
    }
}

void lcd_cmd(unsigned int data)
{	delay_us(1);

  PORTF = data >> 8;				// write high byte
 
  RST_H;

	delay_us(1); 
 	RD_H;  	WR_L;	 RS_L;    CS_L;
	delay_us(2);
	RD_H; 	WR_H;	RS_L;     CS_L;
  PORTF = data & 0xFF;			// write high + low byte 0x00FF
  RST_H;

	delay_us(1); 
 	RD_H;  	WR_L;	 RS_L;    CS_L;
	delay_us(2);
	RD_H; 	WR_H;	RS_L;     CS_L;

}

void lcd_data(unsigned int data)
{
	delay_us(1);
  PORTF = data >> 8;				// write high byte
   RST_H;

	delay_us(1); 
	RD_H;   	WR_L;	 	RS_H; 	CS_L;
	delay_us(2); 
	RD_H;   	WR_H;	  	RS_H; 	CS_L; 
  PORTF = data & 0xFF;			// write high + low byte 0x00FF
  RST_H;

	delay_us(1); 
	RD_H;   	WR_L;	 	RS_H; 	CS_L;
	delay_us(2); 
	RD_H;   	WR_H;	  	RS_H; 	CS_L;
	}


void TFT_SET()
{
 	PORTA = 0xFF;	
//	PORTB = 0xFF;
 	PORTC = 0xFF;	
 //	PORTD = 0xFF;
  	PORTF = 0xFF;

 	delay_ms(120);
 	PORTA = 0x00;	
//	PORTB = 0x00;
 	PORTC = 0x00;
 //	PORTD = 0x00;
 	PORTF = 0x00;
	delay_us(1);
        BL_H;
///////////////////////////////

 	
//	PORTB  &= ~_BV(2);//  L
    RST_L;
    delay_ms(5);
//  PORTB  |= _BV(2);  //  H
    RST_H;
    delay_ms(5); 
//	PORTB  &= ~_BV(2);//  L
    RST_L;
    delay_ms(5);  
//  PORTB  |= _BV(2);  //  H
    RST_H;
    delay_ms(5); 
 BL_H
/////////////////////////////////
// Before power supply startup setting
 lcd_cmd(0x0111);lcd_data(0x1000);
 lcd_cmd(0x0110);lcd_data(0x0100);
 delay_ms(1); 

 lcd_cmd(0x0111);lcd_data(0x000c);
 lcd_cmd(0x0110);lcd_data(0x0101);
 delay_ms(1); 

 lcd_cmd(0x0111);lcd_data(0x1000);
 lcd_cmd(0x0110);lcd_data(0x0101);
 delay_ms(1); 
 lcd_cmd(0x0111);lcd_data(0x1202);
 lcd_cmd(0x0110);lcd_data(0x0102);
 delay_ms(1);

// Power supply startup(1) setting
 lcd_cmd(0x0100);lcd_data(0x4010);

 lcd_cmd(0x0111);lcd_data(0x00bc);
 lcd_cmd(0x0110);lcd_data(0x0100);
delay_ms(1);
 lcd_cmd(0x0111);lcd_data(0x1200);
 lcd_cmd(0x0110);lcd_data(0x0100);
delay_ms(1);
 lcd_cmd(0x0111);lcd_data(0x001c);
 lcd_cmd(0x0110);lcd_data(0x0101);
 delay_ms(35);

// Power supply startup(2) setting
 lcd_cmd(0x0111);lcd_data(0x1a00);
 lcd_cmd(0x0110);lcd_data(0x0100);
delay_ms(1);
// Other mode settings
 lcd_cmd(0x0111);lcd_data(0x0200);
 lcd_cmd(0x0110);lcd_data(0x0103);
delay_ms(1);
 lcd_cmd(0x0111);lcd_data(0x1506);
 lcd_cmd(0x0110);lcd_data(0x0103);
delay_ms(1);
 lcd_cmd(0x0111);lcd_data(0x0c41);
 lcd_cmd(0x0110);lcd_data(0x0104);
 delay_ms(75);

// Display control register setting *** datasheet 16 page
 lcd_cmd(0x0001);lcd_data(0x0000);   
 lcd_cmd(0x0002);lcd_data(0x0700);
 lcd_cmd(0x0003);lcd_data(0x0030);   
 lcd_cmd(0x0008);lcd_data(0x0506);
 lcd_cmd(0x0009);lcd_data(0x0001);
 lcd_cmd(0x000b);lcd_data(0x0000);
 lcd_cmd(0x000d);lcd_data(0x0010);
 lcd_cmd(0x0012);lcd_data(0x0000);
 lcd_cmd(0x0013);lcd_data(0x0002);
 lcd_cmd(0x0015);lcd_data(0x0000);
 lcd_cmd(0x001b);lcd_data(0x0010);
 lcd_cmd(0x001c);lcd_data(0x0000);

// Tset register setting
  lcd_cmd(0x0205);lcd_data(0x0000);
  lcd_cmd(0x070a);lcd_data(0x7800);
  lcd_cmd(0x070b);lcd_data(0x0000);
  lcd_cmd(0x070c);lcd_data(0x0030);
  lcd_cmd(0x070d);lcd_data(0x0000);
  lcd_cmd(0x070e);lcd_data(0x0000);
  lcd_cmd(0x070f);lcd_data(0x2100);
  lcd_cmd(0x0710);lcd_data(0x0000);

// Address setting
 lcd_cmd(0x0400);lcd_data(0x0031);
 lcd_cmd(0x0401);lcd_data(0x0000);
 lcd_cmd(0x0402);lcd_data(0x0000);
 lcd_cmd(0x0403);lcd_data(0x018f);
 lcd_cmd(0x0404);lcd_data(0x0000);

// Y register setting
 lcd_cmd(0x0300);lcd_data(0x0503);
 lcd_cmd(0x0301);lcd_data(0x0403);
 lcd_cmd(0x0302);lcd_data(0x0404);
 lcd_cmd(0x0303);lcd_data(0x0303);
 lcd_cmd(0x0304);lcd_data(0x0302);
 lcd_cmd(0x0305);lcd_data(0x0203);
 lcd_cmd(0x0306);lcd_data(0x0e14);
 lcd_cmd(0x0307);lcd_data(0x0503);
 lcd_cmd(0x0308);lcd_data(0x0403);
 lcd_cmd(0x0309);lcd_data(0x0404);
 lcd_cmd(0x030a);lcd_data(0x0303);
 lcd_cmd(0x030b);lcd_data(0x0302);
 lcd_cmd(0x030c);lcd_data(0x0203);
 lcd_cmd(0x030d);lcd_data(0x1f10);

// Memory access
 lcd_cmd(0x0210);lcd_data(0x0000);// Horizontal Start Address ( y = 0  )
 lcd_cmd(0x0211);lcd_data(0x00ef);// Horizontal End Address   ( y = 239)
 lcd_cmd(0x0212);lcd_data(0x0000);// Vertical Start Address   ( x = 0  )
 lcd_cmd(0x0213);lcd_data(0x018f);// Vertical End Address     ( x = 399)

 lcd_cmd(0x0200);lcd_data(0x0000);// Horizontal Address       ( y = 0  )
 lcd_cmd(0x0201);lcd_data(0x0000);// Vertical Address         ( x = 0  )
}


void LCD_ON()
{
lcd_cmd(0x0100);lcd_data(0x4110);
lcd_cmd(0x0111);lcd_data(0x061c);
lcd_cmd(0x0110);lcd_data(0x0101);
lcd_cmd(0x0007);lcd_data(0x0001);
delay_ms(45);
lcd_cmd(0x0111);lcd_data(0x161b);
lcd_cmd(0x0110);lcd_data(0x0101);
lcd_cmd(0x0007);lcd_data(0x0003);
delay_ms(45);
lcd_cmd(0x0007);lcd_data(0x0113);
}

/*


// BLACK;0x0000, WHITE;0xffff,YELLOW;0xffe0,  RED;0xf800, GREEN;0x07e0,
// BLUE;0x001f,  CYAN;0x07ff, MAGENTA; 0xf81f

// ===============================   ===============================
// ex => mainscreen(GREEN);
// -----------------------------------------------------------------------------
void mainscreen(unsigned color)
{
	screen_color = color;					// ڻ 
}

// =============================== ڻ  ===============================
// ex => textcolor(BLUE);
// -----------------------------------------------------------------------------
void textcolor(unsigned color)
{
	screen_bkcolor = color;				// ڻ 
}
*/


void LCD_Clear(void)
{  long int i;
   lcd_cmd(0x0200);
   lcd_data(0x0000);   // Horizontal Address
   lcd_cmd(0x0201);
   lcd_data(0x0000);   // Vertical Address

   lcd_cmd(0x0202);   //  RAM Data Write/Read
   for(i=0;i<MAX_DOT;i++)
      lcd_data(BLACK); //test WHITE
}

void LCD_gotoxy(unsigned int data1, unsigned int data2)
{  lcd_cmd(0x0200);
   lcd_data(data2);   // Horizontal Address
   lcd_cmd(0x0201);
   lcd_data(data1);   // Vertical Address
}

void LCD_Dot(unsigned int data)
{  lcd_cmd(0x0202);   //  RAM Data Write/Read
   lcd_data(data);    // data = Dot color
}


void G_d(unsigned int p,unsigned int p1)// p1 color
{
 PORTC = p1 & 0xff;   // Ʈ  AND 
 PORTF = (p1 >> 8) & 0xff;
 PORTA = (p & 0x03)|0x04; // Ʈ  OR 
 
	RD_H; 	WR_L;	RS_H;	CS_L;

	RD_H; 	WR_H;	RS_H;	CS_H;

 //PORTB = 0x28;
 ///PORTB = 0x38;
}


// ============================= ׷ ǥ  ==============================
// [μ] int x,int y
// Item1:  xǥ : θ ǥý(0-319) / θ ǥý(0-239)
// Item2:  yǥ : θ ǥý(0-239) / θ ǥý(0-319)
// []  ȭ(0,0,319,239)
//         ȭ(0,0,239,319)
// -----------------------------------------------------------------------------
void Gotoxy(int x,int y)
{
	if(x >= 0 && x <= screen_xmax){			// X ʰ ˻
		if(y >= 0 && y <= screen_ymax){		// Y ʰ ˻
	                 screen_curx = x;
			screen_cury = y;
		}
	}
}

// ======================= ȭ鿡 ؽƮ ǥ  ===========================
// Item1:  xǥ : θ ǥý(1-20) / θ ǥý(1-10)
// Item2:  yǥ : θ ǥý(1-10) / θ ǥý(1-20)
// []  ȭ(1,1,40,15)
//         ȭ(1,1,15,40)
// [] _screen_curx,_screen_cury
// -----------------------------------------------------------------------------
void Gotoxyt(int x,int y)
{
	int x1 = (x-1) * 16,y1;
	y1 = (y-1) * 24;
	if(x1 >= 0 && x1 <= screen_xmax){
		if(y1 >= 0 && y1 <= screen_ymax){
			screen_curx = x1;
			screen_cury = y1;
		}
	}
}



/*
// =======================  Ʈ  ε ===============================
//  [μ] char c
//  Item1: ȯ Ÿ
//   : ȯ  
// -----------------------------------------------------------------------------
void *GetAscImage(int c)
{
	return AscFont16[c];
}


// ========================= Ʈ     =========================
// [μ] unsigned int	c
// Item1:  
// ------------------------------------------------------------------------------
void Putcht(unsigned int c)
{
int i;
unsigned char *image;
		image = GetAscImage(c);		//   
						
		if(c == 0x0a){	//  ڵ Ÿ
			// ü ȭ ǥ 	
			if(screen_mode == SCREEN_COLUMN_SET){
				screen_xmax = SCREEN_COLUMN_X;// θ ִ밪
				screen_ymax = SCREEN_COLUMN_Y;// θ ִ밪
			}
			else{
				screen_xmax = SCREEN_ROW_X;	// θ ִ밪
				screen_ymax = SCREEN_ROW_Y;	// θ ִ밪
			}
			// YĿ 
			screen_cury += 16;			// Ŀ   
			if(screen_cury > screen_ymax) screen_cury = 0;// Ŀ   ˻
		}
		else if(c == 0x0d) screen_curx = 0;		// Carrage Return
		else{
			image = GetAscImage(c);		//   
		
		}


}



// ============================== ѹڸ  Ѵ. ==========================
// [μ] unsigned c
// Item1:  (ѱ ڵ )
// -----------------------------------------------------------------------------
void Putch(unsigned c)
{
	Putcht(c&0xff);						// 1Ʈ ڸ ǥ
}


// ================  ׷ ǥ ѹڸ  Ѵ. ===================
// [μ] int x
//	  int y
//	  unsigned c
// Item1:    ġ(0 - 319)
// Item2:    ġ(0 - 239)
// Item3:  (ѱ ڵ )
// -----------------------------------------------------------------------------
void Putchxy(int x,int y,unsigned c)
{
	Gotoxy(x,y);						// Base Cursor Point Set
	Putcht(c);						// 1Ʈ ڸ ǥ
}



// ===================  ؽƮ ǥ ѹڸ  Ѵ. ================
// Item1:  xǥ : θ ǥý(1-20) / θ ǥý(1-10)
// Item2:  yǥ : θ ǥý(1-10) / θ ǥý(1-20)
// Item3:  (ѱ ڵ )
// -----------------------------------------------------------------------------
void Putchxyt(int x,int y,unsigned c)
{
	Gotoxyt(x,y);						// Base Cursor Point Set
	Putch(c);						// 1Ʈ ڸ ǥ
}

*/


/*
 
// ====================  Ŀ ġ  ڿ  Ѵ. ==================
// [μ] const char *str
// Item1:  ڿ(ѿ)
// -----------------------------------------------------------------------------
void Puts(const char *str)
{
unsigned char byte1st = 0,c;
unsigned datau,i = 0;

	Han_Conv((char*)str);					// ϼ  ȯ
	while((c = han_conv_buf[i++]) != 0){			// ҽ Han_Conv() 
//	while((c = *str++) != 0){				// 쿡 ϼ->  
		if(byte1st){
			datau = ((unsigned)byte1st << 8) | (unsigned)c;
			_Putch(datau);				// ѱ ڸ ǥ
			byte1st = 0;                    	
		}                                       	
		else if(c & 0x80){                      	
			byte1st = c;				// ѱ Ÿ 
		}                                       	
		else{                                   	
			if(c == 0x0a){				//  ǵ
				_Putch(0x0d);           	
				_Putch(0x0a);           	
			}                               	
			else _Putch(c); 			//  ڸ ǥ
		}
	}
}

*/





// =============================== ȭ鿡   ============================
// [μ] int x,int y,unsigned color
// Item1:  xǥ(0-319)
// Item2:  yǥ(0-219)
// Item3: ǥ (_RED,_GREEN,BLUE,_WHITE,_BLACK...)
// []  ȭ(0,0,319,239)
//         ȭ(0,0,239,319)
// -----------------------------------------------------------------------------
void Putpixel(int x,int y,unsigned color)
{
//	ScreenSet(screen_mode,x,y,x,y);			// ǥ ȭ 


lcd_cmd(0x0200);
   lcd_data(y);   // Horizontal Address
   lcd_cmd(0x0201);
   lcd_data(x);   // Vertical Address

  lcd_cmd(0x0202);   //  RAM Data Write/Read
   lcd_data(color);    // data = Dot color
}


void tftlcdSetPixel (unsigned int x,unsigned int y, unsigned int color)
{
	 if(x>(X_MAX_PIXEL -1))
	  x = (X_MAX_PIXEL -1);

	 if(y>(Y_MAX_PIXEL-1))
	  y = (Y_MAX_PIXEL -1);
	Putpixel(x,y,color);  //pixel =>  Putpixel
	
	
	/*
	 Init_Data(0x0002,startx >> 8);
	 Init_Data(0x0003,startx);
	 Init_Data(0x0004,startx >> 8);
	 Init_Data(0x0005,startx);
	 Init_Data(0x0006,starty >> 8);
	 Init_Data(0x0007,starty);
	 Init_Data(0x0008,starty >> 8);
	 Init_Data(0x0009,starty);
	 WriteI(0x0022);
 
	 sbi(RS_PORT, RS);
	 cbi(NCS_PORT, NCS);
	 outb(LCDLOW_DATAPORT_PORT, (u08)(color & 0x00FF));    	 
   	 outb(LCDHI_DATAPORT_PORT, (u08)(color >> 8));

     cbi(NWR_PORT, NWR);
	 sbi(NWR_PORT, NWR);
	
	 sbi(NCS_PORT, NCS);
*/
}

void tftlcdWriteChar(unsigned int x, unsigned int y, unsigned char c, unsigned int color)
{
	unsigned char i = 0, j = 0;
	unsigned char data;
	for(i=0; i<5; i++)   //5   14
	{
		data = pgm_read_byte(&E_font[((c - 0x20) * 5) + i]);  //5  14
		for(j = 0; j < 8; j++)   //8,10
		{
			if (data & (1 << j))
				tftlcdSetPixel(x + i, y + j, color);
			//else	tftlcdSetPixel(x + i, y + j, 0xffff);  //߰ ϸ ڽȿ ۾ 
		}
	}
}

void tftlcdPutStr(unsigned int x, unsigned int y, unsigned char *data, unsigned int color)
{
	while (*data) {
    	tftlcdWriteChar(x, y, *data, color);
    	data++;
    	x += 8;
    	if (x >= X_MAX_PIXEL)
    	{
    		x = 0;
    		y += 8;      //8,10
    	}
  	}
}








void TFT_screen_mode(U08 mode)			/* set screen direction mode */
{
  if((mode == 'P') || (mode == 'p'))		// portrait mode(30x40 character unit)
    { ScreenMode = 'P';
      LineLimit = 30;
    }
  else						// landscape mode(40x30 character unit)
    { ScreenMode = 'L';
      LineLimit = 40;
    }
}

void TFT_English(U08 code)			/* write a English ASCII character */
{
  unsigned char data, x, y;
  unsigned int pixel[8][16];
  unsigned int dot0, dot1, dot2, dot3, dot4;

  for(x = 0; x < 8; x++)			// read English ASCII font
    { data = pgm_read_byte(&E_font[code][x]);
      for(y = 0; y < 8; y++)
        { if(data & 0x01) pixel[x][y] = foreground;
	  else            pixel[x][y] = background;
	  data = data >> 1;
	}
    }

  for(x = 0; x < 8; x++)
    { data = pgm_read_byte(&E_font[code][x+8]);
      for(y = 0; y < 8; y++)
	{ if(data & 0x01) pixel[x][y+8] = foreground;
	  else            pixel[x][y+8] = background;
	  data = data >> 1;
	}
    }

  if(outline_flag == 1)				// display outline
    for(x = 0; x < 8; x++)
      { dot0 = pgm_read_byte(&E_font[code][x]) + pgm_read_byte(&E_font[code][x+8])*256;
        dot1 = dot0 >> 1;								 // up side
        dot2 = dot0;									 // down side
        dot3 = pgm_read_byte(&E_font[code][x+1]) + pgm_read_byte(&E_font[code][x+9])*256;// left side
        dot4 = pgm_read_byte(&E_font[code][x-1]) + pgm_read_byte(&E_font[code][x+7])*256;// right side

        for(y = 0; y < 15; y++)
	  { if(!(dot0 & 0x0001))
	      { if(dot1 & 0x0001) pixel[x][y] = outline;
	        if(dot2 & 0x0001) pixel[x][y] = outline;
	        if((dot3 & 0x0001) && (x < 7 )) pixel[x][y] = outline;
	        if((dot4 & 0x0001) && (x > 0 )) pixel[x][y] = outline;
	      }

            dot1 >>= 1;
            dot2 = dot0;
	    dot0 >>= 1;
	    dot3 >>= 1;
	    dot4 >>= 1;
          }
      }

  if((cursor_flag == 1) && (xcharacter == xcursor) &&(ycharacter == ycursor))
    { for(x = 0; x < 8; x++)			// display cursor
        { pixel[x][14] = cursor;
          pixel[x][15] = cursor;
        }
    }

  if(ScreenMode == 'P')
    { for(y = 0; y < 16; y++)			// write in portrait mode
        for(x = 0; x < 8; x++)
          tftlcdSetPixel(xcharacter*8 + x, ycharacter*8 + y, pixel[x][y]);
    }
  else
    { for(y = 0; y < 16; y++)			// write in landscape mode
        for(x = 0; x < 8; x++)
        tftlcdSetPixel((29-ycharacter)*8 + 7 - y, xcharacter*8 + x, pixel[x][y]);
    }

  xcharacter += 1;
  if(xcharacter >= LineLimit)			// end of line ?
    { xcharacter = 0;
      ycharacter += 2;
    }
}




unsigned int KS_code_conversion(U16 KSSM)	/* convert KSSM(ϼ) to KS() */
{
  unsigned char HB, LB;
  unsigned int index, KS;

  HB = KSSM >> 8;
  LB = KSSM & 0x00FF;

  if(KSSM >= 0xB0A1 && KSSM <= 0xC8FE)
    { index = (HB - 0xB0)*94 + LB - 0xA1;
      KS  =  pgm_read_byte(&KS_Table[index][0]) * 256;
      KS |=  pgm_read_byte(&KS_Table[index][1]);

      return KS;
    }
  else
    return -1;
}








void TFT_string(U08 xChar, U08 yChar, U16 colorfore, U16 colorback, char *str)	/* write TFT-LCD string */
{
  unsigned char ch1;
  unsigned int ch2;

  xcharacter = xChar;
  ycharacter = yChar;

  foreground = colorfore;
  background = colorback;

  while(*str)
    { ch1 = *str;
      str++;

      if(ch1 < 0x80)				// English ASCII character
	TFT_English(ch1);
      else					// Korean
	{ ch2 = (ch1 << 8) + (*str);
          str++;
	  ch2 = KS_code_conversion(ch2);	// convert KSSM(ϼ) to KS()
	  TFT_Korean(ch2);
	}
   }
}



void TFT_Korean(U16 code)			/* write a Korean character */
{
  unsigned char cho_5bit, joong_5bit, jong_5bit;
  unsigned char cho_bul, joong_bul, jong_bul = 0, i, jong_flag;
  unsigned int ch;

  cho_5bit   = pgm_read_byte(&table_cho[(code >> 10) & 0x001F]);  // get 5bit(14-10) of chosung
  joong_5bit = pgm_read_byte(&table_joong[(code >> 5) & 0x001F]); // get 5bit(09-05) of joongsung
  jong_5bit  = pgm_read_byte(&table_jong[code & 0x001F]);	  // get 5bit(04-00) of jongsung

  if(jong_5bit == 0)				// if jongsung not exist
    { jong_flag = 0;
      cho_bul = pgm_read_byte(&bul_cho1[joong_5bit]);
      if((cho_5bit == 1) || (cho_5bit == 16))
   	joong_bul = 0;
      else
	joong_bul = 1;
    }
  else						// if jongsung exist
    { jong_flag = 1;
      cho_bul = pgm_read_byte(&bul_cho2[joong_5bit]);
      if((cho_5bit == 1) || (cho_5bit == 16))
	joong_bul = 2;
      else
	joong_bul = 3;
      jong_bul = pgm_read_byte(&bul_jong[joong_5bit]);
    }

  ch = cho_bul*20 + cho_5bit;			// get chosung font 
  for(i = 0; i < 32; i++)
    KoreanBuffer[i] = pgm_read_byte(&K_font[ch][i]);
  ch = 8*20 + joong_bul*22 + joong_5bit;	// OR joongsung font
  for(i = 0; i < 32; i++)
    KoreanBuffer[i] |= pgm_read_byte(&K_font[ch][i]);
  if(jong_flag)					// OR jongsung font
    { ch = 8*20 + 4*22 + jong_bul*28 + jong_5bit;
      for(i = 0; i < 32; i++)
	KoreanBuffer[i] |= pgm_read_byte(&K_font[ch][i]);
    }

  unsigned char data, x, y;
  unsigned int pixel[16][16];
  unsigned int dot0, dot1, dot2, dot3, dot4;

  for(x = 0; x < 16; x++)			// read Korean font
    { data = KoreanBuffer[x];
      for(y = 0; y < 8; y++)
	{ if(data & 0x01) pixel[x][y] = foreground;
	  else            pixel[x][y] = background;
	  data = data >> 1;
	}
    }

  for(x = 0; x < 16; x++)
    { data = KoreanBuffer[x+16];
      for(y = 0; y < 8; y++)
	{ if(data & 0x01) pixel[x][y+8] = foreground;
	  else            pixel[x][y+8] = background;
	  data = data >> 1;
	}
    }

  if(outline_flag == 1)				// display outline
    for(x = 0; x < 16; x++)
      { dot0 = KoreanBuffer[x] + KoreanBuffer[x+16]*256;
        dot1 = dot0 >> 1;				   // up side
        dot2 = dot0;					   // down side
        dot3 = KoreanBuffer[x+1] + KoreanBuffer[x+17]*256; // left side
        dot4 = KoreanBuffer[x-1] + KoreanBuffer[x+15]*256; // right side

        for(y = 0; y < 16; y++)
	  { if(!(dot0 & 0x0001))
	      { if(dot1 & 0x0001) pixel[x][y] = outline;
	        if(dot2 & 0x0001) pixel[x][y] = outline;
 	        if((dot3 & 0x0001) && (x < 15 )) pixel[x][y] = outline;
	        if((dot4 & 0x0001) && (x > 0 ))  pixel[x][y] = outline;
	      }

	    dot1 >>= 1;
	    dot2 = dot0;
	    dot0 >>= 1;
	    dot3 >>= 1;
	    dot4 >>= 1;
          }
      }

  if((cursor_flag == 1) && (xcharacter == xcursor) &&(ycharacter == ycursor))
    { for(x = 0; x < 16; x++)			// display cursor
        { pixel[x][14] = cursor;
          pixel[x][15] = cursor;
        }
    }

  if(xcharacter >= (LineLimit-1))		// end of line ?
    { xcharacter = 0;
      ycharacter += 2;
    }

  if(ScreenMode == 'P')				// write in portrait mode
    { for(y = 0; y < 16; y++)
        for(x = 0; x < 16; x++)
          tftlcdSetPixel(xcharacter*8 + x, ycharacter*8 + y, pixel[x][y]);
    }
  else						// write in landscape mode
    { for(y = 0; y < 16; y++)
        for(x = 0; x < 16; x++)
         tftlcdSetPixel((29-ycharacter)*8 + 7 - y, xcharacter*8 + x, pixel[x][y]);
    }

  xcharacter += 2;
  if(xcharacter >= LineLimit)			// end of line ?
    { xcharacter = 0;
      ycharacter += 2;
    }
}

void TFT_cursor(U16 cursor_color)		/* set cursor and color */
{
  if(cursor_color == Transparent)		// disable cursor
    cursor_flag = 0;
  else						// enable cursor
    { cursor_flag = 1;
      cursor = cursor_color;
    }
}

void TFT_outline(U16 outline_color)		/* set outline and color */
{
  if(outline_color == Transparent)		// disable outline
    outline_flag = 0;
  else						// enable outline
    { outline_flag = 1;
      outline = outline_color;
    }
}

























//#define X_MAX_PIXEL  400 #define Y_MAX_PIXEL  240

/*

void tftlcdImageWrite (unsigned int startx,unsigned int starty,unsigned int imageWidth,unsigned int imageHeight, const unsigned int *image)
{
	 unsigned int i,j;
	 unsigned int data, cnt = 0; 	
	 unsigned int endx, endy;

	endx = startx + imageWidth - 1;
	endy = starty + imageHeight - 1;
	
	//Putpixel(startx,starty,color);  //pixel =>  Putpixel
lcd_cmd(0x0200);
   lcd_data(starty);   // Horizontal Address
   lcd_cmd(0x0201);
   lcd_data(startx);   // Vertical Address
lcd_cmd(0x0202);   //  RAM Data Write/Read

	 for(i = 0;i < imageHeight; i++)
	 {
	     for(j= 0; j< imageWidth; j++)
	     {  
	     	data = pgm_read_word(&image[cnt]);	
		// lcd_data(color);    // data = Dot color
		//outb(LCDLOW_DATAPORT_PORT, (u08)(data & 0x00FF)); 	 
		//outb(LCDHI_DATAPORT_PORT, (u08)(data >> 8));
		  //cbi(NWR_PORT, NWR);
		  //sbi(NWR_PORT, NWR);
lcd_cmd(0x0200);
   lcd_data(starty);   // Horizontal Address
   lcd_cmd(0x0201);
   lcd_data(startx);   // Vertical Address
		
 lcd_cmd(0x0202);   //  RAM Data Write/Read
   lcd_data(cnt);    // data = Dot color
	    cnt++;
	     }
	} 
     
 //	  sbi(NCS_PORT, NCS);

}

*/





void ScreenSet(int mode,int x1,int y1,int x2,int y2)
{

	if(mode == SCREEN_COLUMN_SET){
	lcd_cmd(0x0200);
	//	CLCD_Command_Data(0x16,0x00a8);			// Memory Access Contro : MV=1, MX=0, MY=1, BGR=1 mode.
		screen_mode = SCREEN_COLUMN_SET;		// ȭ  ġHorizontal
		screen_xmax = x2;				// θ ִ밪
		screen_ymax = y2;				// θ ִ밪
	}
	else{
lcd_cmd(0x0202);   //  RAM Data Write/Read
	//	CLCD_Command_Data(0x16,0x00c8);			// Memory Access Contro : MV=1, MX=1, MY=1, BGR=1 mode.
		screen_mode = SCREEN_ROW_SET;			// ȭ  ġVertical
		screen_xmax = x2;				// θ ִ밪
		screen_ymax = y2;				// θ ִ밪
	}


// Memory access
 lcd_cmd(0x0210);lcd_data(0x0000);// Horizontal Start Address ( y = 0  )
 lcd_cmd(0x0211);lcd_data(0x00ef);// Horizontal End Address   ( y = 239)
 lcd_cmd(0x0212);lcd_data(0x0000);// Vertical Start Address   ( x = 0  )
 lcd_cmd(0x0213);lcd_data(0x018f);// Vertical End Address     ( x = 399)
/*
	CLCD_Command_Data(0x02,(x1>>8)&0xff);			// Column Address Start Register SC[8..15]
	CLCD_Command_Data(0x03,x1&0xff);			// Column Address Start Register SC[0..7]

	CLCD_Command_Data(0x06,(y1>>8)&0xff);			// Row Address Start Register SP[8..15]
	CLCD_Command_Data(0x07,y1&0xff);			// Row Address Start Register SP[0..7]

	CLCD_Command_Data(0x04,(x2>>8)&0xff);			// Column Address End Register EC[8..15]
	CLCD_Command_Data(0x05,x2&0xff);			// Column Address End Register EC[0..7]

	CLCD_Command_Data(0x08,(y2>>8)&0xff);			// Row Address End Register EP[8..15]
	CLCD_Command_Data(0x09,y2&0xff);			// Row Address End Register EP[0..7]

*/

}





// ============== TFT LCD  ġ                ___________
// ex1 => Line(xǥ,yǥ , xǥ , yǥ    |           |
// ex2 => Line(50,120,100,200,RED);                               |           |
// -------------------------------------------------------         -----------
void Line(int x1,int y1,int x2,int y2, unsigned color)
{
int i,vdx,vdy,vix,viy,vinc,plot,plotx,ploty,x = 0,y = 0;

	vdx = x2 - x1;						// vdx = x2 - x1
	vdy = y2 - y1;						// vdy = y2 - y1
	vix = abs(vdx);						// vix = |vdx|
	viy = abs(vdy); 					// vix = |vdx|
	if(vix > viy) vinc = vix;				// vinc = vix ? viy;
	else vinc = viy;

	plotx = x1;
	ploty = y1;
	Putpixel(plotx,ploty,color);  //pixel =>  Putpixel
	for(i = 0; i <= vinc; i++){
		x += vix;
		y += viy;
		plot = 0;
		if(x > vinc){
			x -= vinc;
			if(vdx > 0) plotx++;
			if(vdx < 0) plotx--;
			plot = 1;
		}
		if(y > vinc){
			y -= vinc;
			if(vdy > 0) ploty++;
			if(vdy < 0) ploty--;
			plot = 1;
		}
		if(plot)Putpixel(plotx,ploty,color);
	}
	//if(screen_mode == SCREEN_COLUMN_SET) ScreenSet(SCREEN_COLUMN_SET,0,0,SCREEN_COLUMN_X,SCREEN_COLUMN_Y);// ǥȭ  
	//else
    //                                     ScreenSet(SCREEN_ROW_SET,0,0,SCREEN_ROW_X,SCREEN_ROW_Y);// ǥȭ  
}







 void Rectangle(int x1,int y1,int x2,int y2, unsigned color)
{
	Line( x1, y1, x2, y1, color);
	Line( x2, y1, x2, y2, color);
	Line( x1, y2, x2, y2, color);
	Line( x1, y1, x1, y2, color);


}



void red_set(void)
{
unsigned long dd;

for(dd=0;dd<96000;dd++){
G_d(0x03,0xffff); //0x03 => color 3ŭ Ʈ 
}
delay_ms(100);	delay_ms(500);
for(dd=0;dd<96000;dd++){
G_d(0x00,0x0000);  //0
}
delay_ms(100);	delay_ms(500);
for(dd=0;dd<96000;dd++){
G_d(0x00,0x003f); //0
}
delay_ms(100);	delay_ms(500);
for(dd=0;dd<96000;dd++){
G_d(0x00,0x0fc0); //0
}

delay_ms(100);	delay_ms(500);
for(dd=0;dd<96000;dd++){
G_d(0x03,0xf000);
}

delay_ms(100);
for(dd=0;dd<96000;dd++){
G_d(0x1,0x2222);
}
delay_ms(100);
for(dd=0;dd<96000;dd++){
G_d(0x01,0x6666);
}
delay_ms(100);
for(dd=0;dd<96000;dd++){
G_d(0x00,0xacac);
}
delay_ms(100);
for(dd=0;dd<96000;dd++){
G_d(0x00,0x1111);
}
delay_ms(100);
}
// ============ 16bit ̹  ǥ(ȭ ü Ǵ ū ȭ) ===============
// [μ] int mode,int x,int y,int x_size,int y_size,unsigned char *image1,unsigned char *image2,unsigned char *image3
// Item1: ȭ   :: _SCREEN_COLUMN_SET( ȭ), _SCREEN_ROW_SET( ȭ)
// Item2:  xǥ : θ ǥý(0-319) / θ ǥý(0-239)
// Item3:  yǥ : θ ǥý(0-239) / θ ǥý(0-319)
// Item4: x ̹ ũ
// Item5: y ̹ ũ
// Item6: ǥ ̹ 1(32000Byte ̳)
// Item7: ǥ ̹ 2(32000Byte ̳)
// Item8: ǥ ̹ 3(32000Byte ̳)
// Item9: ǥ ̹ 4(32000Byte ̳)
// Item10:ǥ ̹ 5(32000Byte ̳)
// []  ȭ(0,0,319,239)
//        RGB(565) : RRRRRGGGGGGBBBBB
// -----------------------------------------------------------------------------
void Image_Screen_Disp_16bit(int mode,int x, int y, int x_size,int y_size,unsigned char *image1,unsigned char *image2,unsigned char *image3,unsigned char *image4,unsigned char *image5)
{
unsigned char *image;
unsigned i,j,total_cnt = 0,total_size;
long total_sizel;

	ScreenSet(mode,x,y,x+(x_size-1),y+(y_size-1));		// ǥȭ(/) 
	total_sizel = (long)x_size * (long)y_size;		// ü ǥ  
	if(total_sizel >= 32000L){
		total_cnt = (unsigned)(total_sizel / 32000L);	// LOOP ī 
		i = (unsigned)(total_sizel % 32000L);		// ?
		if(i != 0) total_cnt++;
		total_size = 32000U;				//  			
	}
	else if(total_sizel > 0){				//  0 ũ
		total_size = (unsigned)total_sizel; 
		total_cnt = 1;
	}
	//CLCD_Command(0x22);					// Set LCD register address 0x22 for make the system take GRAM data.
	//PORTE |= 0x20;						// PORTE.5 :: CLCD RS = 1 :: Data Set
	//PORTG &= ~0x04;						// PORTG.2 :: CLCD_CS = 0
	//PORTG |= 0x01;						// PORTG.0 :: CLCD_WR\ = 1
	lcd_cmd(0x0202);   //  RAM Data Write/Read
	RS_H;
	CS_L;
    WR_H;

		
	for(i = 0; i < total_cnt; i++){				// ü  ?
		if(i > 0){					// ù°  
			if(total_sizel >= 32000L){		// ǥ  
				total_size = 32000;
				total_sizel -= 32000L;
			}
			else total_size = (unsigned)total_sizel; 
		}
	
	switch(i){
			case 0: image = image1;			// ̹ 1
				break;
			case 1: image = image2;			// ̹ 2
				break;
			case 2: image = image3;			// ̹ 3
				break;
			case 3: image = image4;			// ̹ 4
				break;
			case 4: image = image5;			// ̹ 5
				break;
			default: break;
		}

	
		for(j = 0; j < total_size; j++){		// ü (64000Byte ̳)
			PORTC = pgm_read_byte(image++);		// Data Read D[8..15]
			PORTF = pgm_read_byte(image++);		// Data Read D[0..7]
	
		//	PORTG &= ~0x01;				// PORTG.0 :: CLCD_WR\ = 0
		//	PORTG |= 0x01;				// PORTG.0 :: CLCD_WR\ = 1
	WR_L;
	WR_H;
		}
	}
//	PORTG |= 0x04;						// PORTG.2 :: CLCD_CS = 1
	CS_H;

}



void draw_bird0(void)
{
	unsigned char i = 0, j = 0;
	unsigned char data;
//CLCD_Command(0x22);	

for(i=0;i<2960;i++) //5   14
	{
		data = pgm_read_byte(i);  //5  14
		for(j = 0; j < 320; j++)   //8,10
		{
			if (data & (1 << j))
		
	
	data = pgm_read_byte(j++);	
		
		
			//	tftlcdSetPixel(x + i, y + j, color);
			//else	tftlcdSetPixel(x + i, y + j, 0xffff);  //߰ ϸ ڽȿ ۾ 
		}
	}
}










void Demo_display_8_color(void)
{
   unsigned int row, idx, cnt;

   lcd_cmd(0x0202);
      
   // Display Direction (Top->Down)

   for(row=0;row<X_MAX_PIXEL;row++)
      for(idx=0;idx<8;idx++)
         for(cnt=0;cnt<Y_MAX_PIXEL/8;cnt++)
            lcd_data(basic_color[idx]);
}


void Demo_display_16_gray(void)
{
   unsigned int row, idx, cnt;

   lcd_cmd(0x0202);
      
   // Display Direction (Top->Down)

   for(row=0;row<X_MAX_PIXEL;row++)
      for(idx=0;idx<16;idx++)
         for(cnt=0;cnt<Y_MAX_PIXEL/16;cnt++)
            lcd_data(gray16[idx]);
}


